"""
class ConfigurationWarning

A warning that is raised when a configuration issue is detected in the mqkit library. This
could include issues such as missing or invalid configuration parameters
"""

from .mqkitwarning import MqkitWarning


class ConfigurationWarning(MqkitWarning):
    """
    class ConfigurationWarning

    A warning that is raised when a configuration issue is detected in the mqkit library. This
    could include issues such as missing or invalid configuration parameters
    """
