"""
module mqkit.warnings.mqkitwarning

Defines the MqkitWarning class for general warnings related to the mqkit library.
"""


class MqkitWarning(UserWarning):
    """
    class MqkitWarning

    Parent class for all warnings related to the mqkit library. Inherits from UserWarning.
    """
