"""
module mqkit.events

Defines the AppEventType enumeration for application event types that
can result in callbacks being invoked.
"""

__all__ = [
    "AppEventType",
]

from .appeventtype import AppEventType
